#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
# -----------------------------------------------------------------------------------------------
# THIS PLUGIN WAS DEVELOPED AND TESTED UNDER WINDOWS VISTA ONLY AND MAY OR MAY NOT WORK ON A MAC. 
# -----------------------------------------------------------------------------------------------
#    Name:	Multiple Offsets
#      By:	sdmitch
#   Usage:	Create multiple offsets to a selected face
#		 Note:	This version does not address the issue of overlap of "In" offsets.
#						Working from a face rather than selected edges is so much easier.
#    Date:	Feb 2012
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
if not file_loaded?(File.basename(__FILE__))
    UI.menu('Plugins').add_item('Multiple Offsets') { SDM_Multiple_Offsets.do_offsets }
	 file_loaded(File.basename(__FILE__))
end
#
module SDM_Multiple_Offsets

	def self.do_offsets
		mod=Sketchup.active_model
		ent=mod.active_entities
		sel=mod.selection
		unless sel.empty?
			@cnt = 1 if !@cnt; @dir = "In" if !@dir; @osd = 1.inch if !@osd
			inp=UI.inputbox([" Number:","Direction:","   Offset:"],[@cnt,@dir,@osd],[nil,"In|Out|Both",nil],"Multiple Offsets")
			if !inp then;return;end
			mod.start_operation("Bevel Maker",true)
			@cnt = inp[0].to_i; @dir = inp[1]; @osd=inp[2].to_l; faces=[];
			osd = @osd; osd = -osd if @dir == "In"
			ent.each{|f| faces<<f if f.is_a?(Sketchup::Face)}# collect all existing faces
			for i in 1..@cnt
				sel.each{|f| 
					if f.is_a?(Sketchup::Face)
						f.offset(-osd) if @dir == "Both"
						f.offset(osd)
					end
				}# offset all selected faces
				ent.each{|f| 
					if f.is_a?(Sketchup::Face)
						if sel.include?(f)
							ent.erase_entities(f) # erase 
						else# invert selection
							sel.toggle f  if !faces.include?(f)
						end
					end
				}
			end
			ent.erase_entities(sel)
			mod.commit_operation
		else
			UI.messagebox "Nothing selected.  Select faces tobe offset"
		end
	end

	# Code copied from offset.rb
	# Copyright 2004,2005,2006,2009 by Rick Wilson - All Rights Reserved

	class Sketchup::Face
		def offset(dist)
			begin
				pi = Math::PI
				if (not ((dist.class==Fixnum || dist.class==Float || dist.class==Length) && dist!=0))
					return nil
				end
				verts=self.outer_loop.vertices;pts = [];vecs = []
				verts.reverse! if self.outer_loop.edges[0].reversed_in?(self)
				# CREATE ARRAY pts OF OFFSET POINTS FROM FACE
				0.upto(verts.length-1) do |a|
					vec1 = (verts[a].position-verts[a-(verts.length-1)].position).normalize
					vec2 = (verts[a].position-verts[a-1].position).normalize
					vec3 = (vec1+vec2).normalize
					if vec3.valid?
						ang = vec1.angle_between(vec2)/2
						ang = pi/2 if vec1.parallel?(vec2)
						vec3.length = dist/Math::sin(ang)
						vecs << vec3
						t = Geom::Transformation.new(vec3)
						if pts.length > 0
							vec4 = pts.last.vector_to(verts[a].position.transform(t))
							if vec4.valid?
								unless (vec2.parallel?(vec4))
									t = Geom::Transformation.new(vec3.reverse)
								end
							end
						end
						pts.push(verts[a].position.transform(t))
					else
						puts "vec3 is invalid"
					end
				end
				# CHECK FOR DUPLICATE POINTS IN pts ARRAY
				duplicates = []
				pts.each_index do |a|
					pts.each_index do |b|
						next if b==a
						duplicates<<b if pts[a]===pts[b]
					end
					break if a==pts.length-1
				end
				duplicates.reverse.each{|a| pts.delete(pts[a])}
				# CREATE FACE FROM POINTS IN pts ARRAY
				(pts.length > 2) ? (parent.entities.add_face(pts)) : (return nil)
			rescue
				puts "#{self} did not offset: #{pts}"
				return nil
			end
		end
	end

end
